
"""
gpm_download_day_V06B.py
Tools to download daily data downloaded from NASA GPM mission website (This version is for the V06 of GESDIS).

The files in this tool are a modified version of the PPTs tool presented here: https://github.com/lapig-ufg/PPTs

Authors: Marina Ruiz Sánchez-Oro, Guillaume Goodwin
Date: 03/02/2022
"""

#http://stackoverflow.com/questions/10875215/python-urllib-downloading-contents-of-an-online-directory
#http://stackoverflow.com/questions/4589241/downloading-files-from-an-http-server-in-python
#http://stackoverflow.com/questions/22676/how-do-i-download-a-file-over-http-using-python
#http://stackoverflow.com/questions/34831770/download-a-file-in-python-with-urllib.request-instead-of-urllib
#http://stackoverflow.com/questions/25501090/how-to-get-wget-command-info-with-python

################################################################################
################################################################################
"""Import Python packages"""
################################################################################
################################################################################


from urllib.request import urlopen
import http.cookiejar as cookielib

import sys
import os
import re
import string
import datetime
import urllib.request

from Login_UI import retrieveLogin

def gpm_day_download(input_dir, Start_Date = None,End_Date = None, backslh ='\\'):


    # This is my auto-login
    GetLoginInfo = list(retrieveLogin())

    #Get actual time
    try:
        Start_Date = list(map(int,Start_Date.split('-')))
        start_year = int(Start_Date[0])
        start_month = int(Start_Date[1])
        start_day = int(Start_Date[2])
    except:
        Start_Date = ['2000','06','01']
        start_year = int(Start_Date[0])
        start_month = int(Start_Date[1])
        start_day = int(Start_Date[2])
    try:
        End_Date =  list(map(int,(End_Date).split('-')))
        end_year = int(End_Date[0])
        end_month = int(End_Date[1])
        end_day = int(End_Date[2])
    except:
        End_Date = list(map(int,((datetime.datetime.now()).strftime('%Y-%m-%d')).split('-')))
        end_year = int(End_Date[0])
        end_month = int(End_Date[1])
        end_day = int(End_Date[2])


    str_Start_Date = list(map(str,Start_Date))
    str_End_Date = list(map(str,End_Date))



    #Start month
    if len(str_Start_Date[1]) == 1:
        str_Start_Date[1] = '0' + str_Start_Date[1]
    #End month
    if len(str_End_Date[1]) == 1:
        str_End_Date[1] = '0' + str_End_Date[1]

    #Start day
    if len(str_Start_Date[2]) == 1:
        str_Start_Date[2] = '0' + str_Start_Date[2]
    #End day
    if len(str_End_Date[2]) == 1:
       str_End_Date[2] = '0' + str_End_Date[2]


    #print str_Start_Date
    years = list(map(str,range(start_year,end_year+1)))

    #Download files
    try:
        for i in range(0,len(years),1):

            month = []
            months = []

            month = list(map(str,range(1,13)))

            months = list(map(lambda x: '0' + x if len(x)==1 else x,month))

            if years[i] == str_Start_Date[0]:
                try:
                    del months[:months.index(str_Start_Date[1])]
                except:
                    pass

            if years[i] == str_End_Date[0]:
                try:
                    del months[months.index(str_End_Date[1])+1:]
                except:
                    pass

            for j in range(0,len(months),1):
                #print months[j]
                url ='https://gpm1.gesdisc.eosdis.nasa.gov/data/GPM_L3/GPM_3IMERGDF.06/'+years[i]+'/'+months[j]+'/'
                print (url)

                #quit()
                #Acess the URL
                try:
                    urlpath =urlopen(url)
                except:
                    continue
                #Decode the URL
                string = urlpath.read().decode('utf-8')

                #Extract HDF5 files and make an file list
                pattern = re.compile('3B.*?nc4.*?')
                filelist = list(set(list(map(str,pattern.findall(string)))))
                filelist.sort()
                #print (filelist)


                try:
                    try:
                        startImg = filelist.index('3B-DAY.MS.MRG.3IMERG.'+str_Start_Date[0]+str_Start_Date[1]+str_Start_Date[2]+'-S000000-E235959.V06.nc4')
                    except:
                        startImg = filelist.index('3B-DAY.MS.MRG.3IMERG.20000601-S000000-E235959.V06.nc4')
                except:
                    startImg = None

                #DEL UNDER START
                if years[i] == str_Start_Date[0]:
                    if months[j] == str_Start_Date[1]:
                        #Start month
                        try:
                            del filelist[:startImg]
                        except:
                            pass
                    else:
                        pass
                else:
                    pass

                try:
                    endImg = filelist.index('3B-DAY.MS.MRG.3IMERG.'+str_End_Date[0]+str_End_Date[1]+str_End_Date[2]+'-S000000-E235959.V06.nc4')

                except:
                    endImg = None

                #DEL OVER END
                if years[i] == str_End_Date[0]:
                    if months[j] == str_End_Date[1]:
                        #End month
                        try:
                            del filelist[endImg+1:]
                        except:
                            pass
                else:
                    pass

                filteredList = filelist #= list(filter(lambda x: x not in os.listdir(input_dir),filelist))

                #print(filteredList)

                for item in range(0,len(filteredList)):
                    os.system('wget --user=' + GetLoginInfo[0] + ' --password=' + GetLoginInfo[1] + ' --show-progress -c -q '+  url + filteredList[item] + ' -O ' + input_dir + backslh + filteredList[item])

                    #os.system('wget --user=' + os.environ["NASA_USERNAME"] + ' --password=' + os.environ["NASA_PASSWORD"] + ' --show-progress -c -q '+  url + filteredList[item] + ' -O ' + input_dir + backslh + filteredList[item])

    except:
        print ('\nDownloads finished')
    print ('\nDownloads finished')



"""
Add_forecast.py

Authors: Guillaume Goodwin
"""

################################################################################
################################################################################
"""Import Python packages"""
################################################################################
################################################################################

import os
import sys
import argparse
import time
import shutil
import re
import numpy
import tkinter
from tkinter import filedialog
import platform
import argparse
import datetime
from osgeo import gdal, ogr, osr
from osgeo.gdalnumeric import *
from osgeo.gdalconst import *
import csv
import pandas as pd


################################################################################
################################################################################
"""Import internal modules"""
################################################################################
################################################################################

#GPM MONTH
from gpm_download_month_V06B import gpm_month_download
#GPM DAY
from gpm_download_day_V06B import gpm_day_download
#GPM 30min
from gpm_download_30min_V06B import gpm_30min_download

#AncillaryData
from image_process import process
#from get_info import get_info
import General_functions as fn

################################################################################
################################################################################
"""Parse Arguments"""
################################################################################
################################################################################

args = fn.parse_add_rainfall_Arguments()

arglist = [args.base_r,args.supp_r,args.output_f]
print('Found the files to combine!')



# Read in the rainfall file and write to arrays
base_rain = pd.read_csv(arglist[0])
rain_time = base_rain['duration_s'].to_numpy()
rain_intensity = base_rain['intensity_mm_sec'].to_numpy()

# Read in the supplemental rainfall file
supp_rain = pd.read_csv(arglist[1])

# Check they are in the same same time durations
if supp_rain['duration_s'][0] == rain_time[0]:
    print("Both durations are the same, {} seconds".format(str(rain_time[0])))
else:
    print("The durations are different, base time is: {} and supplemental time is {} did you intend this?".format(str(rain_time[0]),str(supp_rain['duration_s'][0])))

# Append the new data
for i in range(0,len(supp_rain)):
    rain_time = numpy.append(rain_time,supp_rain['duration_s'][i])
    rain_intensity = numpy.append(rain_intensity,supp_rain['intensity_mm_sec'][i])
    print(i)

output_data =numpy.column_stack([rain_time, rain_intensity])
# Now save the stuff
with open(arglist[2], "w", newline="") as f:
	    writer = csv.writer(f)
	    writer.writerow(['duration_s','intensity_mm_sec'])
	    writer.writerows(output_data)



"""
gpm_download_day_V06B.py
Tools to download daily data downloaded from NASA GPM mission website (This version is for the V06 of GESDIS).

The files in this tool are a modified version of the PPTs tool presented here: https://github.com/lapig-ufg/PPTs

Authors: Marina Ruiz Sánchez-Oro, Guillaume Goodwin
Date: 03/02/2022
"""



################################################################################
################################################################################
"""Import Python packages"""
################################################################################
################################################################################

import os
import sys
import argparse
import time
import shutil
import re
import numpy
import glob
import tkinter
from tkinter import filedialog
import platform
import argparse
import datetime
from osgeo import gdal, ogr, osr
from osgeo.gdalnumeric import *
from osgeo.gdalconst import *
import csv

################################################################################
################################################################################
"""Import internal modules"""
################################################################################
################################################################################

#GPM MONTH
from gpm_download_month_V06B import gpm_month_download
#GPM DAY
from gpm_download_day_V06B import gpm_day_download
#GPM 30min
from gpm_download_30min_V06B import gpm_30min_download

#AncillaryData
from image_process import process
#from get_info import get_info
import General_functions as fn


################################################################################
################################################################################
"""Argument Parser"""
################################################################################
################################################################################

def parseArguments():

	parser = argparse.ArgumentParser(prog='Precipitation Processing Tool')

	parser.add_argument('--ProdTP', choices= ['GPM_M','GPM_D','GPM_30min'], default='GPM_30min', dest='ProdTP',  help='GPM_M: GPM Monthly (IMERGM v6);\n GPM_D: GPM Daily (IMERGDF v6); \n GPM_30min: GPM Half-hourly (IMERGHHE v6)\n')

	StartDF = '01-06-2000'
	parser.add_argument('--StartDate',dest='StartDate', help='Insert the start date',default=StartDF,type=str)

	EndDF = str((datetime.datetime.now()).strftime('%Y-%m-%d'))
	parser.add_argument('--EndDate',dest='EndDate', help='Insert the end date',default=EndDF,type=str)

	parser.add_argument('--ProcessDir',dest='ProcessDir', help='Insert the processing directory path',type=str)

	parser.add_argument('--SptSlc',dest='SptSlc', nargs="?", help='Insert the slice feature path',type=str)

	parser.add_argument('--OP', dest='OP',action="store_true", help='Call this argument if you only want to process the data. Make sure you have a directory with a raw files subfolder.')

	args = parser.parse_args();
	return args


################################################################################
################################################################################
"""Argument Parser"""
################################################################################
################################################################################

def parse_add_rainfall_Arguments():

	parser = argparse.ArgumentParser(prog='Precipitation Processing Tool')

	parser.add_argument('--base_rainfall', dest='base_r', help='Insert the pathway to the base rainfall file',type=str)

	parser.add_argument('--supp_rainfall', dest='supp_r', help='Insert the pathway to the supplemental rainfall file',type=str)

	parser.add_argument('--output_file', dest='output_f', help='Insert the pathway to the output file',type=str)

	args = parser.parse_args();
	return args


################################################################################
################################################################################
def ENVI_raster_binary_to_2d_array(file_name):
    """
    This function transforms a raster into a numpy array.
    Args:
        file_name (ENVI raster): the raster you want to work on.
        gauge (string): a name for your file
    Returns:
        image_array (2-D numpy array): the array corresponding to the raster you loaded
        pixelWidth (geotransform, inDs) (float): the size of the pixel corresponding to an element in the output array.
    Source: http://chris35wills.github.io/python-gdal-raster-io/
    """


    driver = gdal.GetDriverByName('ENVI')

    driver.Register()

    inDs = gdal.Open(file_name, GA_ReadOnly)

    if inDs is None:
        print ("Couldn't open this file: " + file_name)
        print ("Perhaps you need an ENVI .hdr file? ")
        sys.exit("Try again!")
    else:
        print ("%s opened successfully" %file_name)

        #print '~~~~~~~~~~~~~~'
        #print 'Get image size'
        #print '~~~~~~~~~~~~~~'
        cols = inDs.RasterXSize
        rows = inDs.RasterYSize
        bands = inDs.RasterCount

        #print "columns: %i" %cols
        #print "rows: %i" %rows
        #print "bands: %i" %bands

        #print '~~~~~~~~~~~~~~'
        #print 'Get georeference information'
        #print '~~~~~~~~~~~~~~'
        geotransform = inDs.GetGeoTransform()
        originX = geotransform[0]
        originY = geotransform[3]
        pixelWidth = geotransform[1]
        pixelHeight = geotransform[5]

        #print "origin x: %i" %originX
        #print "origin y: %i" %originY
        #print "width: %2.2f" %pixelWidth
        #print "height: %2.2f" %pixelHeight

        # Set pixel offset.....
        #print '~~~~~~~~~~~~~~'
        #print 'Convert image to 2D array'
        #print '~~~~~~~~~~~~~~'
        band = inDs.GetRasterBand(1)
        #print band
        image_array = band.ReadAsArray(0, 0, cols, rows)
        image_array_name = file_name
        #print type(image_array)
        #print image_array.shape

        return image_array, pixelWidth, (geotransform, inDs)



def maps_to_timeseries(arglist, working_dir, data_product):
	# List the .bil files
	print('Got to the timeseries function')
	files = sorted(os.listdir(working_dir)); bilfiles = []

	for i in range(len(files)):
		ending = files[i][-4:]
		print(f"i am the ending: {ending}")
		if ending == '.bil':
			bilfiles.append(files[i])
	print(bilfiles)


	#Timelist = []
	#CumTimelist = []
	#Intlist = []
	Full_list = []
	thirty_one_day_months = [1,3, 5, 7, 8, 10, 12]
	thirty_day_months = [4, 6, 9, 11]
	february_special_month = [2]
	for i in range(len(bilfiles)):
		print (bilfiles[i])
		leap_year = False
		file_ending = bilfiles[i].split('/')[-1]
		if file_ending[-8:] == '_cut.bil':
			timer = datetime.datetime.strptime(bilfiles[i], 'Calib_rainfall_%Y%m%d-S%H%M%S-V06B_cut.bil')
			month = timer.month
			year = timer.year
			if year % 4 == 0 and (year % 100 != 0 or year % 400 == 0):
				leap_year = True
		else:
			print('I am not the right file')
			pass
		# Add to the rainfall list
		Rainarr, pixelWidth, (geotransform, inDs) = ENVI_raster_binary_to_2d_array(working_dir + bilfiles[i])
		print(Rainarr)
		# takes the average over the whole raster image
		Rain = numpy.mean(Rainarr)
		if data_product == 'GPM_30min':
			converting_factor = 30*60 # seconds in 30 minutes
		elif data_product == 'GPM_D':
			converting_factor = 24*3600 # seconds in a day
		else:
			if month in thirty_day_months:
				converting_factor = 30*24*3600 # seconds in a month
				print('I have 30 days')
			elif month in thirty_one_day_months:
				converting_factor = 31*24*3600 # seconds in a month
				print('I have 31 days')
			else:
				if leap_year == True:
					print('I am february in a leap year ')
					converting_factor = 29*24*3600 # seconds in a month
				else:
					print('I am february in a non-leap year')
					converting_factor = 28*24*3600 # seconds in a month
		Intensity = Rain/(converting_factor) # Intensity of rainfall during the period (mm/sec) - converted from mm
		#Intlist.append(Intensity)

		# Save it all together
		Full_list.append([converting_factor, Intensity])

	# Now save the stuff
	with open(working_dir+arglist[1]+"_to_"+arglist[2]+"_"+arglist[0]+"_rainfall.csv", "w", newline="") as f:
	    writer = csv.writer(f)
	    writer.writerow(['duration_s','rainfall_mm_sec'])
	    writer.writerows(Full_list)
	print ('DOOOOONE')
	print (working_dir)


def move_files(path_to_target, start_date, end_date):
	files_to_move = []
	path_to_bil_file_list = glob.glob(path_to_target + '*bil' )
	path_to_hdr_file_list = glob.glob(path_to_target + '*hdr' )
	path_to_nc4_file_list = glob.glob(path_to_target + '*nc4' )
	path_to_hdf5_file_list = glob.glob(path_to_target + '*hdf5' )
	files_to_move.extend(path_to_bil_file_list)
	files_to_move.extend(path_to_hdr_file_list)
	files_to_move.extend(path_to_nc4_file_list)
	files_to_move.extend(path_to_hdf5_file_list)
	new_directory = path_to_target+'run_start_'+start_date+'_end_'+end_date

	# Check whether the specified path exists or not
	isExist = os.path.exists(new_directory)
	if not isExist:
		# Create a new directory because it does not exist
		os.mkdir(new_directory)
		print("The new directory is created!")

	for path_to_file in files_to_move:
		file_name = os.path.basename(path_to_file)
		shutil.move(path_to_file, new_directory+'/'+file_name)




################################################################################
################################################################################

def download_months(arglist, zero_list, zero_dir, fst_dir, backslh, n):
	if zero_list[n].endswith('.HDF5') > -1 and zero_list[n].find('.xml') == -1 and zero_list[n].find('.aux') == -1 and zero_list[n].find('.tfw') == -1:
			if 	zero_list[n].find('.HDF5') > -1:
				#extract_subdata = 'HDF5:"%s%s%s"://Grid/precipitation' % (zero_dir,backslh,zero_list[n])
				extract_subdata = "%s%s%s" % (zero_dir,backslh,zero_list[n])
				outfile = '%s%s%s.tif' % (fst_dir,backslh,zero_list[n][:-5])
				print(f'this is the outfile: {outfile}')

				process(outfile,extract_subdata,arglist[0])
				raster_crop(arglist, outfile)
				extract_subdata = outfile = None


def download_days(arglist, zero_list, zero_dir, fst_dir, backslh, n):
	if zero_list[n].endswith('.nc4') > -1 and zero_list[n].find('.xml') == -1 and zero_list[n].find('.aux') == -1 and zero_list[n].find('.tfw') == -1:
			#extract_subdata = 'HDF5:"%s%s%s"://precipitationCal' % (zero_dir, backslh, zero_list[n])
			extract_subdata = "%s%s%s" % (zero_dir,backslh,zero_list[n])
			print(f'this is extract_subdata: {extract_subdata}')
			outfile = '%s%s%s_precipitationCal.tif' % (fst_dir,backslh, zero_list[n][:-4])
			print(f'this is the outfile: {outfile}')

			process(outfile,extract_subdata,arglist[0])
			raster_crop(arglist, outfile)
			extract_subdata = outfile = None



def download_hhs(arglist, zero_list, zero_dir, fst_dir, backslh, n):
	if zero_list[n].endswith('.HDF5') > -1 and zero_list[n].find('.xml') == -1 and zero_list[n].find('.aux') == -1 and zero_list[n].find('.tfw') == -1:
		if 	zero_list[n].find('.HDF5') > -1:
			extract_subdata = "%s%s%s" % (zero_dir,backslh,zero_list[n])
			#extract_subdata = 'HDF5:"%s%s%s"://Grid/precipitation' % (zero_dir,backslh,zero_list[n])
			outfile = '%s%s%s.bil' % (fst_dir,backslh,zero_list[n][:-5])
			print(f'this is the outfile: {outfile}')

			process(outfile,extract_subdata,arglist[0])
			raster_crop(arglist, outfile)
			extract_subdata = outfile = None



########
# Crop the raster
########
def raster_crop(arglist, outfile):
	if arglist[4] != 'None':
		cutfile = arglist[4]
		to_cut = outfile
		print(f'cutfile file: {cutfile}')
		print(f'to_cut file: {to_cut}')
		cutted_file = outfile[:-4] + '_cut.bil'
		print(f'cutted file: {cutted_file}')

		# Find out the file path
		f_path_str = cutted_file.rsplit('/',1)[0]
		print(f'f_name {f_path_str}')
		f_path = f_path_str+'/'
		print(f'f_path {f_path}')

		# Create the file name from existing file
		f_name_str = cutted_file.rsplit('/',1)[-1]
		f_name_str_split = f_name_str.split('.')
		print(f_name_str_split)
		date_str = f_name_str_split[4][:16]
		print(f'date_str {date_str}')
		end_string = f_name_str_split[6]
		print(f'end_string {end_string}')
		# the end results of the cutted file should be:
		## A = split the file. we want only the stuff on the first chunk - this will have path info
		## AA = 1 (name directory where we find the data)
		## AAA = /1/ (add the slashes to make up the path)
		## B = 20180101 (the date) - we also want to include the time if it's 30m
		## BB = 'V06B_cut' - the final extension before the file type.



		cutted_file = f_path + 'Calib_rainfall_' + date_str + '-' + end_string + '.bil'

		# Cut the raster to your desired extent
		os.system('gdalwarp -overwrite -of ENVI -t_srs EPSG:4326 -cutline ' + cutfile + ' -crop_to_cutline ' + to_cut + ' ' + cutted_file)
		# Get rid of the big files
		os.system('rm ' + outfile)
		os.system('rm ' + cutted_file+'.aux.xml')
		os.system('rm ' + outfile[:-4] + '.hdr')



"""
Login_UI.py
Set up the login window for entering the NASA password and username details.

The files in this tool are a modified version of the PPTs tool presented here: https://github.com/lapig-ufg/PPTs

Authors: Guillaume Goodwin, Marina Ruiz Sánchez-Oro
Date: 03/02/2022
"""


################################################################################
################################################################################
"""Import Python packages"""
################################################################################
################################################################################

import sys
import os
import platform
import webbrowser
from PyQt5 import QtGui, QtPrintSupport , QtWidgets, QtCore
from PyQt5.QtWidgets import QApplication

def system_os():
	if platform.system() == 'Windows':
		return 1
	else:
		return 2

if system_os() == 1:
	backslh = '\\'

else:
	backslh = '/'


try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s


class Ui_LoginWindow(object):

    def setupUi(self, LoginWindow):
        LoginWindow.setObjectName("LoginWindow")
        LoginWindow.setWindowModality(QtCore.Qt.NonModal)
        LoginWindow.setEnabled(True)
        LoginWindow.resize(357, 161)
        icon = QtGui.QIcon()
        icon_location =  str(os.getcwd())+backslh+'EXE'+backslh+'GPM_PY_EXE'+backslh+'avatar.ico'
        icon.addPixmap(QtGui.QPixmap(icon_location), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        LoginWindow.setWindowIcon(icon)
        LoginWindow.setModal(False)

        self.InfoMessage = QtWidgets.QLabel(LoginWindow)
        self.InfoMessage.setGeometry(QtCore.QRect(20, 10, 293, 20))
        self.InfoMessage.setObjectName("InfoMessage")

        self.LoginLine = QtWidgets.QLineEdit(LoginWindow)
        self.LoginLine.setGeometry(QtCore.QRect(80, 40, 231, 20))
        self.LoginLine.setObjectName("LoginLine")

        self.PasswordLine = QtWidgets.QLineEdit(LoginWindow)
        self.PasswordLine.setGeometry(QtCore.QRect(80, 70, 231, 20))
        self.PasswordLine.setEchoMode(QtWidgets.QLineEdit.Password)
        self.PasswordLine.setObjectName("PasswordLine")

        self.buttonBox = QtWidgets.QDialogButtonBox(LoginWindow)
        self.buttonBox.setGeometry(QtCore.QRect(110, 100, 151, 32))
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")

        self.LoginMessage = QtWidgets.QLabel(LoginWindow)
        self.LoginMessage.setGeometry(QtCore.QRect(20, 40, 61, 20))
        self.LoginMessage.setObjectName("LoginMessage")

        self.PasswordMessage = QtWidgets.QLabel(LoginWindow)
        self.PasswordMessage.setGeometry(QtCore.QRect(20, 70, 61, 20))
        self.PasswordMessage.setObjectName("PasswordMessage")

        self.label = QtWidgets.QLabel(LoginWindow)
        self.label.setGeometry(QtCore.QRect(20, 140, 228, 21))
        self.label.setObjectName("RegisterPhrase")

        self.ClickHere = QtWidgets.QPushButton(LoginWindow)
        self.ClickHere.setGeometry(QtCore.QRect(234, 140, 61, 20))
        self.ClickHere.setObjectName("ClickHere")
        self.ClickHere.setStyleSheet("background-color: #2b90f5")
        self.ClickHere.clicked.connect(self.GoRegister)


        self.retranslateUi(LoginWindow)
        self.buttonBox.accepted.connect(LoginWindow.accept)
        self.buttonBox.rejected.connect(LoginWindow.reject)
        QtCore.QMetaObject.connectSlotsByName(LoginWindow)
        LoginWindow.setTabOrder(self.LoginLine, self.PasswordLine)

    def retranslateUi(self, LoginWindow):
        _translate = QtCore.QCoreApplication.translate
        LoginWindow.setWindowTitle(_translate("LoginWindow", "NASA EarthData Login"))
        self.InfoMessage.setText(_translate("LoginWindow", "Please insert your NASA EarthData Username and Password:"))
        self.LoginMessage.setText(_translate("LoginWindow", "Username:"))
        self.PasswordMessage.setText(_translate("LoginWindow", "Password:"))
        self.label.setText(_translate("LoginWindow", "If you don\'t have an account or can\'t sing in"))
        self.ClickHere.setText(_translate("LoginWindow", "Click Here!"))

    def GoRegister(self):
        #https://urs.earthdata.nasa.gov/approve_app?client_id=e2WVk8Pw6weeLUKZYOxvTQ
        #return webbrowser.open('https://urs.earthdata.nasa.gov/')
        return webbrowser.open('https://urs.earthdata.nasa.gov/approve_app?client_id=e2WVk8Pw6weeLUKZYOxvTQ')

def retrieveLogin():
	app = QtWidgets.QApplication(sys.argv)
	LoginWindow = QtWidgets.QDialog()
	ex = Ui_LoginWindow()
	ex.setupUi(LoginWindow)
	LoginWindow.show()
	result = LoginWindow.exec_()

	if result == 1:
		text = (ex.LoginLine.text(),ex.PasswordLine.text())
		#print("Login in NASA HTTPS successful")
		return map(str,text)
	else:
		return None
	#sys.exit(app.exec_())



#ttp://stackoverflow.com/questions/10875215/python-urllib-downloading-contents-of-an-online-directory
#ttp://stackoverflow.com/questions/4589241/downloading-files-from-an-http-server-in-python
#ttp://stackoverflow.com/questions/22676/how-do-i-download-a-file-over-http-using-python
#ttp://stackoverflow.com/questions/34831770/download-a-file-in-python-with-urllib.request-instead-of-urllib
#ttp://stackoverflow.com/questions/25501090/how-to-get-wget-command-info-with-python


"""
This version is for the V06 of GESDIS
"""

from urllib.request import urlopen
import http.cookiejar as cookielib

import sys
import os
import re
import string
import datetime
import urllib.request

from Login_UI import retrieveLogin

def gpm_30min_download(input_dir, Start_Date = None,End_Date = None, backslh ='\\'):

    print ('started the 30min data download')


    # Login!
    GetLoginInfo = list(retrieveLogin())

    #Get actual time
    try:
        Start_Date = list(map(int,Start_Date.split('-')))
        start_year = int(Start_Date[0])
        start_month = int(Start_Date[1])
        start_day = int(Start_Date[2])
    except:
        Start_Date = ['2000','06','01']
        start_year = int(Start_Date[0])
        start_month = int(Start_Date[1])
        start_day = int(Start_Date[2])
    try:
        End_Date =  list(map(int,(End_Date).split('-')))
        end_year = int(End_Date[0])
        end_month = int(End_Date[1])
        end_day = int(End_Date[2])
    except:
        End_Date = list(map(int,((datetime.datetime.now()).strftime('%Y-%m-%d')).split('-')))
        end_year = int(End_Date[0])
        end_month = int(End_Date[1])
        end_day = int(End_Date[2])


    str_Start_Date = list(map(str,Start_Date))
    str_End_Date = list(map(str,End_Date))



    #Start month
    if len(str_Start_Date[1]) == 1:
        str_Start_Date[1] = '0' + str_Start_Date[1]
    #End month
    if len(str_End_Date[1]) == 1:
        str_End_Date[1] = '0' + str_End_Date[1]

    #Start day
    if len(str_Start_Date[2]) == 1:
        str_Start_Date[2] = '0' + str_Start_Date[2]
    #End day
    if len(str_End_Date[2]) == 1:
       str_End_Date[2] = '0' + str_End_Date[2]


    #print str_Start_Date
    years = list(map(str,range(start_year,end_year+1)))

    #Download files
    #try:
    for i in range(0,len(years),1):

        y = int(years[i])

        day1 = datetime.datetime(y,1,1)
        dayx = datetime.datetime(y,12,31)

        if y == start_year: day1 = datetime.datetime(y,start_month,start_day)
        if y == end_year: dayx = datetime.datetime(y,end_month,end_day)

        start_d = day1-datetime.datetime(y,1,1); start_d = start_d.days + 1
        end_d = dayx-datetime.datetime(y,1,1); end_d = end_d.days + 1

        day = list(map(str,range(start_d,end_d+1)))
        days = list(map(lambda x: '0' + x if len(x)==1 else x,day))
        days = list(map(lambda x: '0' + x if len(x)==2 else x,days))

        print (days)



        for j in range(len(days)):
            #print months[j]
            url ='https://gpm1.gesdisc.eosdis.nasa.gov/data/GPM_L3/GPM_3IMERGHHE.06/'+years[i]+'/'+days[j]+'/'
            print (url)


            #Acess the URL
            try:
                urlpath =urlopen(url)
            except:
                continue

            #Decode the URL
            string = urlpath.read().decode('utf-8')

            #Extract HDF5 files and make an file list
            pattern = re.compile('3B.*?HDF5.*?')
            filelist = list(set(list(map(str,pattern.findall(string)))))
            filelist.sort()

            filteredList = filelist #= list(filter(lambda x: x not in os.listdir(input_dir),filelist))


            for item in range(0,len(filteredList)):

                os.system('wget --user=' + GetLoginInfo[0] + ' --password=' + GetLoginInfo[1] + ' --show-progress -c -q '+  url + filteredList[item] + ' -O ' + outputDir + backslh + filteredList[item])

    print ('\nDownloads finished')



"""
PPT_UI_RUN.py
User interface tools.

The files in this tool are a modified version of the PPTs tool presented here: https://github.com/lapig-ufg/PPTs

Authors: Guillaume Goodwin, Marina Ruiz Sánchez-Oro
Date: 03/02/2022
"""

################################################################################
################################################################################
"""Import Python packages"""
################################################################################
################################################################################

from PyQt5 import QtGui, QtPrintSupport , QtWidgets, QtCore
from PyQt5.QtWidgets import QApplication, QMessageBox, QWidget, QDesktopWidget, QFileDialog, QTextEdit
from PyQt5.QtCore import QCoreApplication
import sys
import os
import datetime
import time

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_GPM_WINDOW(object):

    def setupUi(self, GPM_WINDOW):

        End_Date = list(map(int,((datetime.datetime.now()).strftime('%Y-%m-%d')).split('-')))
        Year = int(End_Date[0])
        Month = int(End_Date[1])
        Day = int(End_Date[2])


        GPM_WINDOW.setObjectName("GPM_WINDOW")
        GPM_WINDOW.setWindowModality(QtCore.Qt.NonModal)
        GPM_WINDOW.setEnabled(True)
        GPM_WINDOW.resize(350, 449)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(GPM_WINDOW.sizePolicy().hasHeightForWidth())
        GPM_WINDOW.setSizePolicy(sizePolicy)
        GPM_WINDOW.setMinimumSize(QtCore.QSize(350, 410))
        GPM_WINDOW.setMaximumSize(QtCore.QSize(1920, 1080))
        GPM_WINDOW.setSizeIncrement(QtCore.QSize(0, 0))
        GPM_WINDOW.setBaseSize(QtCore.QSize(350, 350))
        self.GPM_Window = QtWidgets.QWidget(GPM_WINDOW)
        self.GPM_Window.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.GPM_Window.setObjectName("GPM_Window")


        #SPATIAL SLICE
        self.Global_Slice_BT = QtWidgets.QRadioButton(self.GPM_Window)
        self.Global_Slice_BT.setGeometry(QtCore.QRect(10, 200, 82, 17))
        self.Global_Slice_BT.setChecked(True)
        self.Global_Slice_BT.setObjectName("Global_Slice_BT")

        self.ScaleSlice = QtWidgets.QButtonGroup(GPM_WINDOW)
        self.ScaleSlice.setObjectName("ScaleSlice")
        self.ScaleSlice.addButton(self.Global_Slice_BT)

        self.Regional_Slice_BT = QtWidgets.QRadioButton(self.GPM_Window)
        self.Regional_Slice_BT.setGeometry(QtCore.QRect(10, 220, 82, 17))
        self.Regional_Slice_BT.setChecked(False)
        self.Regional_Slice_BT.setObjectName("Regional_Slice_BT")
        ##
        self.Regional_Slice_BT.toggled.connect(lambda:self.CheckCheck(self.Regional_Slice_BT))

        self.ScaleSlice.addButton(self.Regional_Slice_BT)
        self.Mask_Insert_BT = QtWidgets.QToolButton(self.GPM_Window)
        self.Mask_Insert_BT.setEnabled(False)
        self.Mask_Insert_BT.setGeometry(QtCore.QRect(30, 260, 51, 21))
        self.Mask_Insert_BT.setCheckable(False)
        self.Mask_Insert_BT.setPopupMode(QtWidgets.QToolButton.DelayedPopup)
        self.Mask_Insert_BT.setArrowType(QtCore.Qt.RightArrow)
        self.Mask_Insert_BT.setObjectName("Mask_Insert_BT")
        self.Mask_Insert_BT.clicked.connect(self.selectSlice)
        #self.ScaleSlice.addButton(self.Mask_Insert_BT)

        self.Mask_Insert_LB = QtWidgets.QLabel(self.GPM_Window)
        self.Mask_Insert_LB.setGeometry(QtCore.QRect(30, 240, 111, 16))
        self.Mask_Insert_LB.setObjectName("Mask_Insert_LB")

        self.Spatial_Slice_LB = QtWidgets.QLabel(self.GPM_Window)
        self.Spatial_Slice_LB.setGeometry(QtCore.QRect(10, 180, 181, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.Spatial_Slice_LB.setFont(font)
        self.Spatial_Slice_LB.setObjectName("Spatial_Slice_LB")

        self.MaskDir_TX = QtWidgets.QLineEdit(self.GPM_Window)
        self.MaskDir_TX.setEnabled(False)
        self.MaskDir_TX.setGeometry(QtCore.QRect(90, 260, 221, 20))
        self.MaskDir_TX.setObjectName("MaskDir_TX")

        #TEMPORAL SLICE
        self.Start_Date_Cal = QtWidgets.QDateEdit(self.GPM_Window)
        self.Start_Date_Cal.setGeometry(QtCore.QRect(70, 100, 81, 22))
        self.Start_Date_Cal.setMinimumDateTime(QtCore.QDateTime(QtCore.QDate(1998, 1, 1), QtCore.QTime(0, 0, 0)))
        self.Start_Date_Cal.setCalendarPopup(True)
        self.Start_Date_Cal.setDate(QtCore.QDate(2014, 3, 12))
        self.Start_Date_Cal.setObjectName("Start_Date_Cal")

        self.D_M_Y_LB1 = QtWidgets.QLabel(self.GPM_Window)
        self.D_M_Y_LB1.setGeometry(QtCore.QRect(70, 80, 81, 16))
        self.D_M_Y_LB1.setObjectName("D_M_Y_LB1")

        self.Start_Date_LB = QtWidgets.QLabel(self.GPM_Window)
        self.Start_Date_LB.setGeometry(QtCore.QRect(10, 100, 61, 16))
        self.Start_Date_LB.setObjectName("Start_Date_LB")

        self.End_Date_Cal = QtWidgets.QDateEdit(self.GPM_Window)
        self.End_Date_Cal.setGeometry(QtCore.QRect(70, 150, 81, 22))
        self.End_Date_Cal.setProperty("showGroupSeparator", False)
        self.End_Date_Cal.setCalendarPopup(True)
        self.End_Date_Cal.setDate(QtCore.QDate(Year, Month, Day))
        self.End_Date_Cal.setObjectName("End_Date_Cal")

        self.D_M_Y_LB2 = QtWidgets.QLabel(self.GPM_Window)
        self.D_M_Y_LB2.setGeometry(QtCore.QRect(70, 130, 81, 16))
        self.D_M_Y_LB2.setObjectName("D_M_Y_LB2")

        self.End_Date_LB = QtWidgets.QLabel(self.GPM_Window)
        self.End_Date_LB.setGeometry(QtCore.QRect(10, 150, 51, 16))
        self.End_Date_LB.setObjectName("End_Date_LB")



        #PRODUCT TYPE
        self.GPM_M_BT = QtWidgets.QRadioButton(self.GPM_Window)
        self.GPM_M_BT.setGeometry(QtCore.QRect(10, 30, 82, 17))
        self.GPM_M_BT.setChecked(True)
        self.GPM_M_BT.setObjectName("GPM_M_BT")

        self.Precipitation_Products = QtWidgets.QButtonGroup(GPM_WINDOW)
        self.Precipitation_Products.setObjectName("Precipitation_Products")
        self.Precipitation_Products.addButton(self.GPM_M_BT)
        self.GPM_D_BT = QtWidgets.QRadioButton(self.GPM_Window)
        self.GPM_D_BT.setGeometry(QtCore.QRect(90, 30, 82, 17))
        self.GPM_D_BT.setObjectName("GPM_D_BT")
        self.Precipitation_Products.addButton(self.GPM_D_BT)
        self._TRMM_M_BT = QtWidgets.QRadioButton(self.GPM_Window)
        self._TRMM_M_BT.setGeometry(QtCore.QRect(160, 30, 82, 17))
        self._TRMM_M_BT.setObjectName("_TRMM_M_BT")
        self.Precipitation_Products.addButton(self._TRMM_M_BT)
        self.TRMM_D_BT = QtWidgets.QRadioButton(self.GPM_Window)
        self.TRMM_D_BT.setGeometry(QtCore.QRect(250, 30, 82, 17))
        self.TRMM_D_BT.setObjectName("TRMM_D_BT")
        self.Precipitation_Products.addButton(self.TRMM_D_BT)

        #PRODUCT CHECKS

        self.GPM_M_BT.toggled.connect(lambda:self.CheckProd(self.GPM_M_BT))
        self.GPM_D_BT.toggled.connect(lambda:self.CheckProd(self.GPM_D_BT))
        self._TRMM_M_BT.toggled.connect(lambda:self.CheckProd(self._TRMM_M_BT))
        self.TRMM_D_BT.toggled.connect(lambda:self.CheckProd(self.TRMM_D_BT))


        self.PPT_TypeLb = QtWidgets.QLabel(self.GPM_Window)
        self.PPT_TypeLb.setGeometry(QtCore.QRect(10, 10, 181, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.PPT_TypeLb.setFont(font)
        self.PPT_TypeLb.setObjectName("PPT_TypeLb")

        self.Date_Slice_LB = QtWidgets.QLabel(self.GPM_Window)
        self.Date_Slice_LB.setGeometry(QtCore.QRect(10, 60, 181, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.Date_Slice_LB.setFont(font)
        self.Date_Slice_LB.setObjectName("Date_Slice_LB")

        self.DW_PC_DIR_LB = QtWidgets.QLabel(self.GPM_Window)
        self.DW_PC_DIR_LB.setGeometry(QtCore.QRect(10, 290, 241, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.DW_PC_DIR_LB.setFont(font)
        self.DW_PC_DIR_LB.setObjectName("DW_PC_DIR_LB")

        self.OutDir_BT = QtWidgets.QToolButton(self.GPM_Window)
        self.OutDir_BT.setGeometry(QtCore.QRect(30, 310, 51, 21))
        self.OutDir_BT.setCheckable(False)
        self.OutDir_BT.setPopupMode(QtWidgets.QToolButton.DelayedPopup)
        self.OutDir_BT.setArrowType(QtCore.Qt.RightArrow)
        self.OutDir_BT.setObjectName("OutDir_BT")

        #self.myTextBox = QtWidgets.QTextEdit()
        self.window = GPM_WINDOW
        self.OutDir_BT.clicked.connect(self.selectOUT)

        self.OutDir_TX = QtWidgets.QLineEdit(self.GPM_Window)
        self.OutDir_TX.setGeometry(QtCore.QRect(90, 310, 221, 20))
        self.OutDir_TX.setObjectName("OutDir_TX")


        #PHOTOS
        self.label = QtWidgets.QLabel(self.GPM_Window)
        self.label.setEnabled(True)
        self.label.setGeometry(QtCore.QRect(160, 60, 181, 121))
        self.label.setText("")
        self.label.setPixmap(QtGui.QPixmap("gpm_1.jpg"))
        self.label.setScaledContents(True)
        self.label.setObjectName("label")

        #STATUSBAR
        self.statusbar = QtWidgets.QStatusBar(GPM_WINDOW)
        self.statusbar.setObjectName("statusbar")

        #RUN EXIT BUTTONS
        self.Run_BT = QtWidgets.QPushButton(self.GPM_Window)
        self.Run_BT.setGeometry(QtCore.QRect(90, 360, 75, 23))
        self.Run_BT.setObjectName("Run_BT")
        self.Run_BT.clicked.connect(self.exec_Processing)

        self.Exit_BT = QtWidgets.QPushButton(self.GPM_Window)
        self.Exit_BT.setGeometry(QtCore.QRect(180, 360, 75, 23))
        self.Exit_BT.setObjectName("Exit_BT")
        self.Exit_BT.clicked.connect(QCoreApplication.instance().quit)

        self.OP_BT = QtWidgets.QCheckBox(self.GPM_Window)
        self.OP_BT.setGeometry(QtCore.QRect(10, 390, 81, 21))
        self.OP_BT.setObjectName("OP_BT")
        self.OP_LB = QtWidgets.QLabel(self.GPM_Window)
        self.OP_LB.setGeometry(QtCore.QRect(10, 410, 271, 21))
        self.OP_LB.setObjectName("OP_LB")


        self.retranslateUi(GPM_WINDOW)
        QtCore.QMetaObject.connectSlotsByName(GPM_WINDOW)

    def retranslateUi(self, GPM_WINDOW,):
        _translate = QtCore.QCoreApplication.translate
        GPM_WINDOW.setWindowTitle(_translate("GPM_WINDOW", "Precipitation Processing Tools"))
        self.Global_Slice_BT.setText(_translate("GPM_WINDOW", "Global"))
        self.Regional_Slice_BT.setText(_translate("GPM_WINDOW", "Regional slice"))
        self.Mask_Insert_BT.setText(_translate("GPM_WINDOW", "..."))
        self.Mask_Insert_LB.setText(_translate("GPM_WINDOW", "Insert the slice mask:"))
        self.D_M_Y_LB1.setText(_translate("GPM_WINDOW", "Day/Month/Year"))
        self.Start_Date_LB.setText(_translate("GPM_WINDOW", "Start Date:"))
        self.D_M_Y_LB2.setText(_translate("GPM_WINDOW", "Day/Month/Year"))
        self.End_Date_LB.setText(_translate("GPM_WINDOW", "End Date:"))
        self.GPM_M_BT.setText(_translate("GPM_WINDOW", "GPM Month"))
        self.GPM_D_BT.setText(_translate("GPM_WINDOW", "GPM Day"))
        self._TRMM_M_BT.setText(_translate("GPM_WINDOW", "TRMM Month"))
        self.TRMM_D_BT.setText(_translate("GPM_WINDOW", "TRMM Day"))
        self.PPT_TypeLb.setText(_translate("GPM_WINDOW", "Precipitation Product type:"))
        self.Date_Slice_LB.setText(_translate("GPM_WINDOW", "Date slice:"))
        self.Spatial_Slice_LB.setText(_translate("GPM_WINDOW", "Spatial slice:"))
        self.DW_PC_DIR_LB.setText(_translate("GPM_WINDOW", "Download and Processing directory:"))
        self.OutDir_BT.setText(_translate("GPM_WINDOW", "..."))
        self.OutDir_TX.setText(_translate("GPM_WINDOW", ''))
        self.MaskDir_TX.setText(_translate("GPM_WINDOW", ''))
        self.Run_BT.setText(_translate("GPM_WINDOW", "Run"))
        self.Exit_BT.setText(_translate("GPM_WINDOW", "Close"))
        self.OP_BT.setText(_translate("GPM_WINDOW", "Only Process"))
        self.OP_LB.setText(_translate("GPM_WINDOW", "(Mark this only if you wanna process downloaded data)"))

    def selectOUT(self):
        outdir = QFileDialog.getExistingDirectory(self.window, 'Select the download and/or processing directory', os.getenv('HOME'))
        outdir_txt = r'%s' % str(outdir);
        self.OutDir_TX.setText(outdir_txt)

    def selectSlice(self):
        dataMask = QFileDialog.getOpenFileName(self.window, 'Select the data you want to use like mask (Shapefiles ONLY!!!!)', os.getenv('HOME'),"ESRI Shapefile (*.shp)")
        dataMask_txt = r'%s' % str(dataMask[0]);
        self.MaskDir_TX.setText(dataMask_txt)
        #return()
    def CheckCheck(self,b):
            if b.isChecked() == True:
                self.Global_Slice_BT.setChecked(False)
                self.Mask_Insert_BT.setEnabled(True)
                self.Mask_Insert_BT.setCheckable(True)
                self.MaskDir_TX.setEnabled(True)
            else:
                self.Global_Slice_BT.setChecked(True)
                self.Mask_Insert_BT.setEnabled(False)
                self.Mask_Insert_BT.setCheckable(False)
                self.MaskDir_TX.setEnabled(False)
                self.MaskDir_TX.setText('')

    def CheckProd(self,a):

        DateNow = list(map(int,((datetime.datetime.now()).strftime('%Y-%m-%d')).split('-')))
        YearNow = int(DateNow[0])
        MonthNow = int(DateNow[1])
        DayNow = int(DateNow[2])

        if a.isChecked() == True and a.text() == "GPM Month":
            self.Start_Date_Cal.setDate(QtCore.QDate(2014, 3, 12))
            self.End_Date_Cal.setDate(QtCore.QDate(YearNow, MonthNow, DayNow))

        elif a.isChecked() == True and a.text() == "GPM Day":
            self.Start_Date_Cal.setDate(QtCore.QDate(2014, 3, 12))
            self.End_Date_Cal.setDate(QtCore.QDate(YearNow, MonthNow, DayNow))

        elif a.isChecked() == True and a.text() == "TRMM Month":
            self.Start_Date_Cal.setDate(QtCore.QDate(1998, 1, 1))
            self.End_Date_Cal.setDate(QtCore.QDate(YearNow, MonthNow, DayNow))

        elif a.isChecked() == True and a.text() == "TRMM Day":
            self.Start_Date_Cal.setDate(QtCore.QDate(1998, 1, 1))
            self.End_Date_Cal.setDate(QtCore.QDate(YearNow, MonthNow, DayNow))

        else:
            pass

    def exec_Processing(self):
        ProdTp = None

        if self.GPM_M_BT.isChecked() == True and self.GPM_M_BT.text() == "GPM Month":
            ProdTp = 'GPM_M'
        elif self.GPM_D_BT.isChecked() == True and self.GPM_D_BT.text() == "GPM Day":
            ProdTp = 'GPM_D'
        elif self._TRMM_M_BT.isChecked() == True and self._TRMM_M_BT.text() == "TRMM Month":
            ProdTp = 'TRMM_M'
        elif self.TRMM_D_BT.isChecked() == True and self.TRMM_D_BT.text() == "TRMM Day":
            ProdTp = 'TRMM_D'
        else:
            pass

        OP_Info = ''
        if self.OP_BT.isChecked() == True:
            OP_Info = '--OP'

        StartDate = str((datetime.datetime.strptime(str(self.Start_Date_Cal.text()),'%d/%M/%Y')).strftime('%Y-%M-%d'))
        EndDate = str((datetime.datetime.strptime(str(self.End_Date_Cal.text()),'%d/%M/%Y')).strftime('%Y-%M-%d'))
        Donwload_Dir = r'%s' % str(self.OutDir_TX.text())

        Slice_Dir = r'%s' % str(self.MaskDir_TX.text())

        if Slice_Dir == '':
            Slice_Dir = 'None'

        #zz = (ProdTp,StartDate,EndDate,Donwload_Dir,Slice_Dir,OP_Info)
        #print(zz);
        os.system('python Integration.py --ProdTP ' + ProdTp + ' --StartDate ' + StartDate + ' --EndDate ' + EndDate + ' --ProcessDir ' + Donwload_Dir + ' --SptSlc ' + Slice_Dir + ' ' + OP_Info)

if __name__ == "__main__":


    app = QtWidgets.QApplication(sys.argv)
    Dialog = QtWidgets.QDialog()
    ui = Ui_GPM_WINDOW()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec_())




"""
gpm_download_month_V06B.py
Tools to download monthly data downloaded from NASA GPM mission website (This version is for the V06 of GESDIS).

The files in this tool are a modified version of the PPTs tool presented here: https://github.com/lapig-ufg/PPTs

Authors: Marina Ruiz Sánchez-Oro, Guillaume Goodwin
Date: 03/02/2022
"""



#http://stackoverflow.com/questions/10875215/python-urllib-downloading-contents-of-an-online-directory
#http://stackoverflow.com/questions/4589241/downloading-files-from-an-http-server-in-python
#http://stackoverflow.com/questions/22676/how-do-i-download-a-file-over-http-using-python
#http://stackoverflow.com/questions/34831770/download-a-file-in-python-with-urllib.request-instead-of-urllib
#http://stackoverflow.com/questions/25501090/how-to-get-wget-command-info-with-python


################################################################################
################################################################################
"""Import Python packages"""
################################################################################
################################################################################


import sys
import os
import re
import string
import datetime
import numpy as np
from urllib.request import urlopen

from Login_UI import retrieveLogin

def what_files_to_keep_case_1(mylist,start_month_download,end_month_download, items_to_keep):
    # for the first year of the data required, given the end date is also in that year and there is only one year to take data from
    for item in range(0,len(mylist)):
        month_of_file = int(mylist[item].split(".")[-3])
        item_name = mylist[item]
        if ((month_of_file >= start_month_download) and (month_of_file <= end_month_download)):
            print(f'i am keeping month{month_of_file}')
            print(f'item_name:{str(item_name)}')
            items_to_keep.append(item_name)
        else:
            print(f'i am not keeping{month_of_file}')
            print(f'item name: {item_name}')
    return items_to_keep


def what_files_to_keep_case_2(mylist, start_month_download, items_to_keep):
    # first year but there are more years to come. Download ends in month 12 instead in End_month.
    for item in range(0,len(mylist)):
        month_of_file = int(mylist[item].split(".")[-3])
        item_name = mylist[item]
        if ((month_of_file >= start_month_download)):
            print(f'i am keeping month{month_of_file}')
            print(f'item_name:{str(item_name)}')
            items_to_keep.append(item_name)
        else:
            print(f'i am not keeping{month_of_file}')
            print(f'item name: {item_name}')
    return items_to_keep

def what_files_to_keep_case_3(mylist,items_to_keep):
    # keep all files in that year - this is the case for an intermediate year in the given time period
    for item in range(0,len(mylist)):
        month_of_file = int(mylist[item].split(".")[-3])
        item_name = mylist[item]
        print(f'i am keeping month{month_of_file}')
        items_to_keep.append(item_name)
    return items_to_keep


def what_files_to_keep_case_4(mylist, end_month_download,items_to_keep):
    # last year of the period. Always starts in month 1 and finishes in end_month
    for item in range(0,len(mylist)):
        month_of_file = int(mylist[item].split(".")[-3])
        item_name = mylist[item]
        if ((month_of_file <= end_month_download)):
            print(f'i am keeping month{month_of_file}')
            print(f'item_name:{str(item_name)}')
            items_to_keep.append(item_name)
        else:
            print(f'i am not keeping{month_of_file}')
            print(f'item name: {item_name}')
    return items_to_keep



def gpm_month_download(outputDir, Start_Date = None,End_Date = None, backslh ='\\'):

    GetLoginInfo = list(retrieveLogin())

    #Get actual time
    try:
        Start_Date = list(map(int,Start_Date.split('-')))
        start_year = int(Start_Date[0])
        start_month = int(Start_Date[1])
        start_day = 1
    except:
        Start_Date = ['2000','06','01']
        start_year = int(Start_Date[0])
        start_month = int(Start_Date[1])
        start_day = int(Start_Date[2])
    try:
        End_Date =  list(map(int,(End_Date).split('-')))
        end_year = int(End_Date[0])
        end_month = int(End_Date[1])
        end_day = 1
    except:
        End_Date = list(map(int,((datetime.datetime.now()).strftime('%Y-%m-%d')).split('-')))
        end_year = int(End_Date[0])
        end_month = int(End_Date[1])
        end_day = int(End_Date[2])


    str_Start_Date = list(map(str,Start_Date))
    str_End_Date = list(map(str,End_Date))
    print('hello')


    #Start month
    if len(str_Start_Date[1]) == 1:
        str_Start_Date[1] = '0' + str_Start_Date[1]
    #End month
    if len(str_End_Date[1]) == 1:
        str_End_Date[1] = '0' + str_End_Date[1]

    #Start day
    if len(str_Start_Date[2]) == 1:
        str_Start_Date[2] = '0' + str_Start_Date[2]
    #End day
    if len(str_End_Date[2]) == 1:
       str_End_Date[2] = '0' + str_End_Date[2]


    years = list(map(str,range(start_year,end_year+1)))
    print(f'years: {years}')
    items_to_keep = []
    start_datetime = datetime.datetime(Start_Date[0], Start_Date[1], Start_Date[2])
    end_datetime = datetime.datetime(End_Date[0], End_Date[1], End_Date[2])
    num_months = (end_datetime.year - start_datetime.year) * 12 + (end_datetime.month - start_datetime.month)
    full_file_list = []
    year_count = 0


    for i in range(0,len(years),1):
        #print(f'I am moving on to year {years[i]}')

        url ='https://gpm1.gesdisc.eosdis.nasa.gov/data/GPM_L3/GPM_3IMERGM.06/'+years[i]+'/'
        print(url)

        #Acess the URL
        try:
            urlpath =urlopen(url)
        except:
            continue

        #Decode the URL
        string = urlpath.read().decode('utf-8')

        #Extract HDF5 files and make a file list
        pattern = re.compile('3B.*?HDF5.*?')
        filelist = list(set(list(map(str,pattern.findall(string)))))
        filelist.sort()

        filteredList = filelist #= list(filter(lambda x: x not in os.listdir(outputDir),filelist))
        # extend to get a full list
        #full_file_list.extend(filteredList)
        start_month_download = int(str_Start_Date[1])
        end_month_download = int(str_End_Date[1])


        months_to_download = np.arange(int(start_month_download), int(end_month_download),1)
        #items_to_keep = []
        #print(f'this is year count {year_count}')
        #print(f'this is the length of years: {len(years)}')
        if (year_count == 0 and len(years)==1):
            # we start from the first year
            to_keep = what_files_to_keep_case_1(filteredList, start_month_download, end_month_download,items_to_keep)
            print('case 1: this is the first year and there are no more years to come')
            to_keep.extend(full_file_list)
        elif (year_count == 0 and len(years)!=1):
            # first year but there are more to come:
            to_keep = what_files_to_keep_case_2(filteredList,start_month_download,items_to_keep)
            to_keep.extend(full_file_list)
            print('case 2: this is the first year and there are more years to come')
        elif (year_count != 0 and len(years)!=(year_count+1)):
            #keep all files
            to_keep = what_files_to_keep_case_3(filteredList,items_to_keep)
            to_keep.extend(full_file_list)
            print('case 3: this is an intermediate year')
        else:
            # last year
            to_keep = what_files_to_keep_case_4(filteredList,end_month_download,items_to_keep)
            to_keep.extend(full_file_list)
            print('case 4: this is the last year')



        print(f'these are the items to keep: {items_to_keep}')
        year_count +=1
        print('Starting download')


        for item in range(0,len(items_to_keep)):
            os.system('wget --user=' + GetLoginInfo[0] + ' --password=' + GetLoginInfo[1] + ' --show-progress -c -q '+  url + filteredList[item] + ' -O ' + outputDir + backslh + filteredList[item])

            #os.system('wget --user=' + os.environ["NASA_USERNAME"] + ' --password=' + os.environ["NASA_PASSWORD"] + ' --show-progress -c -q '+  url + items_to_keep[item] + ' -O ' + outputDir + backslh + items_to_keep[item])

    #except:
        #print ('\nDownloads finished')

    #print ('\nDownloads finished')



"""
image_process.py
Tools to process the data downloaded from NASA GPM mission website.

The files in this tool are a modified version of the PPTs tool presented here: https://github.com/lapig-ufg/PPTs

Authors: Guillaume Goodwin, Marina Ruiz Sánchez-Oro
Date: 03/02/2022
"""
################################################################################
################################################################################
"""Import Python packages"""
################################################################################
################################################################################



import os
import sys
import string
import argparse
import time
import subprocess
import shutil
import string
import numpy
import re
import numpy as np
from osgeo import gdal, ogr, osr
from osgeo.gdalnumeric import *
from osgeo.gdalconst import *


def process(out_dir,data_file,dataInfo):
	print ()
	print ('PROCESSING')
	print (out_dir)
	print (f'file name: {data_file}')
	print (dataInfo)
	print()


	hourFactor = None

	if dataInfo in ['GPM_D','GPM_30min']:
		hourFactor = 1

	elif dataInfo == 'GPM_M':
		# we need to split also the directory
		fileName = (str(data_file).split("\\")[-1]).split('/')[-1]#.split('"')[1].split('/')[-1]
		print(f'this is the filename: {fileName}')


		if str(fileName[20:28]) == '20140312':
			hourFactor = 456

		elif (int(fileName[20:24]) % 4) == 0:
					if (fileName[24:26]) == '02':
						hourFactor = 696

					elif (fileName[24:26]) in ['01','03','05','07','08','10','12']:
						hourFactor = 744

					elif (fileName[24:26]) in ['04','06','09','11']:
						hourFactor = 720

		elif (fileName[24:26]) == '02':
			hourFactor = 672

		elif (fileName[24:26]) in ['01','03','05','07','08','10','12']:

			hourFactor = 744

		elif (fileName[24:26]) in ['04','06','09','11']:
			hourFactor = 720



	fname = str(data_file)
	outname = str(out_dir)
	outfile =  outname #[:-4].replace(".", "")
	data_file_no_extension = os.path.splitext(data_file)[0]
	#outfile = outfile + '.bil'

	#####################################
	# 0. Make the HDF5 file into a .tif containing the calibrated precipitation
	#####################################
	print (f"data file name:{data_file}")
	print(f'This is the data file no extension: {data_file_no_extension}')
	#quit()
	if dataInfo == 'GPM_30min':
		os.system('gdal_translate -of GTiff HDF5:' + data_file + '://Grid/precipitationCal ' + data_file_no_extension + '.tif')
	elif dataInfo == 'GPM_D':
		os.system('gdal_translate -of GTiff HDF5:' + data_file + '://precipitationCal ' + data_file_no_extension + '.tif')
	else:
		os.system('gdal_translate -of GTiff HDF5:' + data_file + '://Grid/precipitation ' + data_file_no_extension + '.tif')

	print('done converting')
	print(' I can do my job up to here. I am not a complete failure yay!')


	#####################################
	# 1. The produced .tif file is rotated 90° so we must rotated back and assign a correct projection
	# While we're at it, let's amke it a .bil file
	#####################################

	print ('transposing and projecting into ', outfile)

	# Open the created .tif
	ds = gdal.Open(data_file_no_extension + '.tif')

	# Find out its transformation
	#gt=ds.GetGeoTransform()

	# Extract the band you need
	band = ds.GetRasterBand(1)
	print('done extracting band')



	arr = band.ReadAsArray()
	TPSGPM = rot90(arr,1)
	TPSGPM = ((TPSGPM.astype(float))> 0)*((TPSGPM.astype(float))*hourFactor) + ((TPSGPM.astype(float))< 0)*0

	# Set the driver
	driver = gdal.GetDriverByName("ENVI")
	print('set the driver')

	# Set the extents and pixel sizes. That will be the tricky part
	x_pixels = ds.RasterYSize  # number of pixels in x
	y_pixels = ds.RasterXSize  # number of pixels in y
	# WHY THE SWITCH? WE ROTATED THE TPSGPM ArrAY, REMEMBER?

	# x_min & y_max are like the "top left" corner. In WGS84, these are:
	x_min = -180; x_max = 180
	y_max = 90; y_min = -90

	# Pixels need to cover the entire extent
	X_PXL_SIZE = (x_max - x_min) / x_pixels # size of the X pixels
	Y_PXL_SIZE = (y_max - y_min) / y_pixels  # size of the Y pixels
	print('calculated raster properties')


	#wkt_projection = 'a projection in wkt that you got from other file'

	# Create the dataset
	dataset = driver.Create(
		outfile,
        x_pixels,
        y_pixels,
        1,
        gdal.GDT_Float32, )
	print('created driver')

	# Define the GeoTransform

	dataset.SetGeoTransform((
        x_min,    # 0
        X_PXL_SIZE,  # 1
        0,                      # 2
        y_max,    # 3
        0,                      # 4
        -Y_PXL_SIZE))



	# Define the target srs
	print('creating srs')
	srs = osr.SpatialReference()
	srs.ImportFromEPSG(4326)
	dataset.SetProjection(srs.ExportToWkt())
	print('writing to band')




	# Write array to band
	dataset.GetRasterBand(1).WriteArray(TPSGPM)
	print('save to disk')

	# Save to disk
	dataset.FlushCache()  # Write to disk.
	print('remove tif file')

	# remove the annoying tif file
	os.system('rm ' + data_file_no_extension + '.tif')
	print('Done removing')
	print('Script image_process.py is working fine')



'''
process_timeseries_files_pipeline.py

Processes precipitation timeseries data from raster files downloaded from the
NASA GPM mission.

Author: Marina Ruiz Sánchez-Oro
Date: 17/01/2022
'''

import numpy as np
import xarray as xr
import rioxarray
import datetime
import re
import sys
import argparse
import glob
import os
import pandas as pd
from shapely.geometry import Point
from shapely.ops import transform
import pyproj

parser = argparse.ArgumentParser()
parser.add_argument("-f", "--file_folder", dest = "file_folder", help="Folder with the files")
parser.add_argument("-c", "--crs", dest = "crs", help="Coordinate system in the format EPSG:XXXX")
parser.add_argument("-x", "--x_lon",dest = "longitude", help="Longitude of point", type=int)
parser.add_argument("-y", "--y_lat",dest ="latitude", help="Latitude of point", type=int)
parser.add_argument("-t", "--time",dest = "time", help="Date time in format %Y-%m-%d:%H%M%S")#, type=int)

args = parser.parse_args()

file_folder = args.file_folder
coordinate = args.crs
x_lon_to_slice = args.longitude
y_lat_to_slice = args.latitude
time_to_slice = args.time
print(time_to_slice)
time_to_slice = datetime.datetime.strptime(time_to_slice, "%Y-%m-%d:%H%M%S")
print(type(time_to_slice))
print(time_to_slice)

# extract all raster files from the given folder
os.chdir(file_folder)
file_names = []
for file in glob.glob("*.bil"):
    file_names.append(file)



print(f'These are the files I am going to process: {file_names}')
print(f'file folder: {file_folder},\
 longitude: {x_lon_to_slice},\
  latitude: {y_lat_to_slice}, \
  full_date: {time_to_slice}')


def sort_file_list(file_list):
    """
    Sort list of files based on date given on the filename.

    Parameters
    ----------
    file_list : list of str
        List of files to sort.

    Returns
    ----------
    file_list_sorted : list of str
        List of sorted files.

    Author: MRSO
    """
    file_list_sorted=[]
    timeformat = "%Y%m%d" # this is how your timestamp looks like
    regex = re.compile("Calib_rainfall_([0-9]{8})-S([0-9]{6})")
    #Calib_rainfall_20140110-S000000-bil
    def gettimestamp(thestring):
        m = regex.search(thestring)
        print(m)
        return datetime.datetime.strptime(m.groups()[0], timeformat)

    for fn in sorted(file_list, key=gettimestamp):
        file_list_sorted.append(fn)
    return file_list_sorted




def extract_datetime_from_file(file_name):
    """
    Extract date from a file name and convert it to a datetime object.

    Parameters
    ----------
    file_name : str
        Name of file to extract date from.

    Returns
    ----------
    date_formatted : datetime
        Date extracted from filename.
    """
    date_file_1 = re.search("([0-9]{8})", file_name)
    hour_file_1 = re.search("(S[0-9]{6})",file_name)
    date_number_1 = date_file_1.group(0)
    hour_file_1 = hour_file_1.group(0)

    year = int(date_number_1[0:4])
    month = int(date_number_1[4:6])
    day = int(date_number_1[6:8])
    hour = int(hour_file_1[1:3])
    minute = int(hour_file_1[3:5])
    second = int(hour_file_1[5:7])
    date_formatted = datetime.datetime(year, month, day, hour, minute, second)
    return date_formatted


def output_precipitation_timeseries(lon, lat, netcdf_filename):
    """
    Extract a precipitation timeseries from a netCDF file given a lat, lon point.

    Parameters
    ----------
    lon : int
        Longitude.
    lat : int
        Latitude.
    netcdf_filename : str
        Name of netCDF (.nc) file to extract date from.

    Returns
    ----------
    precip_timeseries : list of int
        Timeseries of precipitation for the given lat, lon coordinates.
    """
    joint_ds = xr.open_dataset(netcdf_filename, engine="rasterio")
    precip_timeseries = joint_ds.sel(x=lon, y = lat, method="nearest").precipitation.to_numpy().ravel()
    return precip_timeseries

def output_precipitation_raster(time_to_slice, netcdf_filename):
    """
    Slice a netCDF file from a timeslice and create new netCDF file with the sliced data.

    Parameters
    ----------
    time_to_slice : datetime
        Date and time to slice from the data.
    netcdf_filename : str
        Name of netCDF (.nc) file to extract date from.

    Returns
    ----------
    None
    """
    # could potentially increase functionality by adding output data format: netcdf or raster
    joint_ds = xr.open_dataset(netcdf_filename, engine="rasterio")
    sliced_joint_ds = joint_ds.sel(time=time_to_slice).precipitation
    date_string_name = time_to_slice.strftime('%Y%m%d-%H%M%S')
    sliced_joint_ds.to_netcdf(f'output_precipitation_raster_{date_string_name}.nc', mode='w', format='NETCDF3_64BIT')
    #return sliced_joint_ds


def concatenate_raster_files(dataset_names, output_joint_file_name):
    """
    Read from a list of raster files, concatenate them along the time direction\
    and create a netCDF file.

    Parameters
    ----------
    dataset_names : list of str
        Names of the raster files to concatenate.
    output_joint_file_name : str
        Name of output file.

    Returns
    ----------
    joint_ds : xarray dataset
        Concatenated raster files.
    date_list : list of datetime
        Dates corresponding to the raster files.
    """
    joint_ds_list = []
    date_list = []
    for i in range(len(dataset_names)):
        date_file = re.search("([0-9]{8})", dataset_names[i])
        hour_file = re.search("(S[0-9]{6})", dataset_names[i])

        date_file = extract_datetime_from_file(dataset_names[i])

        xds = xr.open_dataset(dataset_names[i], engine="rasterio")
        # the spatial reference is the coordinate system information
        expanded_ds = xds.expand_dims("time").assign_coords(time=("time", [date_file]))
        expanded_ds = expanded_ds.drop('band')
        expanded_ds = expanded_ds.rename({'band_data':'precipitation'})
        joint_ds_list.append(expanded_ds)
        date_list.append(date_file)

    joint_ds = xr.concat(joint_ds_list, dim='time')
    joint_ds['precipitation'].attrs = {'description':'precipitation amount in mm/s'}
    joint_ds.to_netcdf(output_joint_file_name, mode='w', format='NETCDF3_64BIT')

    return joint_ds, date_list

def convert_crs_point(point_x, point_y, in_proj, out_proj):
    """
    Change the coordinate system of a lat, lon point.

    Parameters
    ----------
    point_x : int
        Longitude coordinate.
    point_y : int
        Latitude coordinate.
    in_proj : str
        Coordinate system to transform from.
    out_proj : str
        Coordinate system to transform to.

    Returns
    ----------
    AoI_point : shapely Point
        Lat, lon point in new coordinate system.
    """
    in_pt = Point(point_x, point_y)
    in_proj = pyproj.CRS(in_proj)
    out_proj = pyproj.CRS(out_proj)
    project =  pyproj.Transformer.from_crs(in_proj, out_proj, always_xy=True).transform
    AoI_point = transform(project, in_pt)
    return AoI_point

# first we need to convert the point to the coordinate system that we want.
# need to first check what the coordinate system of the area is



file_names_sorted = sort_file_list(file_names)

print(f'These are the files I am going to concatenate: {file_names}')
output_joint_file_name = 'joint_ds_with_all_times.nc'


joint_ds, date_list = concatenate_raster_files(file_names_sorted, output_joint_file_name)
print(f'I have concatenated all your files and created a time series')

print(f'This is the date list: {date_list}')

# need to make this better as this is not necessarily the closest point
time_selected = joint_ds.sel(time=time_to_slice)

# first we need to convert the point to the coordinate system that we want.
# need to first check what the coordinate system of the area is

joint_ds = xr.open_dataset('joint_ds_with_all_times.nc', engine="rasterio")
raster_crs = joint_ds.rio.crs

converted_lat_lon = convert_crs_point(x_lon_to_slice, y_lat_to_slice, coordinate, raster_crs)
x_converted = round(converted_lat_lon.x, 2)
y_converted = round(converted_lat_lon.y,2)


timeseries=output_precipitation_timeseries(x_converted, y_converted, output_joint_file_name)

timeseries_df = pd.DataFrame(timeseries, columns=['precipitation_mm/s'])
# need to add time datetime column
timeseries_df['date'] = pd.to_datetime(date_list)
timeseries_df = timeseries_df.set_index('date')
timeseries_df = timeseries_df.sort_values(by='date')
print(timeseries_df.head)

timeseries_df.to_csv(f'precipitation_timeseries_point_lon_{x_lon_to_slice}_lat_{y_lat_to_slice}.csv')

output_precipitation_raster(time_to_slice, output_joint_file_name)
